#include <iostream>
#include <stdlib.h>

static const int N = 1000;
static const int STEPS = 1000000;

struct Player
{
    bool isA;
    bool isTolerant;
    int payoff;
    bool *judgesGood;
};

int main(int argc, char **argv)
{
    (void) argc;
    (void) argv;

    // Créer N joueurs qui jugent tous les autres comme étant bons
    Player *players = new Player[N];
    int tolerant_players = 0;

    for (int i=0; i<N; ++i) {
        players[i].isA = rand() & 1;
        players[i].isTolerant = rand() & 1;
        players[i].payoff = 0;
        players[i].judgesGood = new bool[N];

        if (players[i].isTolerant) {
            tolerant_players += 1;
        }

        // Initialiser les jugements
        for (int j=0; j<N; ++j) {
            players[i].judgesGood[j] = rand() & 3;
        }
    }

    // Simuler le jeu
    for (int t=0; t<STEPS; ++t) {
        // Choisir deux joueurs
        int player_a_index = rand() % N;
        int player_b_index = rand() % N;

        Player *player_a = &players[player_a_index];
        Player *player_b = &players[player_b_index];

        // Savoir si le joueur A va coopérer avec le joueur B
        bool cooperate = false;

        if (!player_a->isTolerant) {
            cooperate = (player_a->isA == player_b->isA);
        } else {
            cooperate = (player_a->judgesGood[player_b_index] | player_a->judgesGood[player_a_index]);
        }

        cooperate &= rand() & 7;       // Une chance sur 8 que la coopération échoue

        if (cooperate) {
            player_a->payoff -= 2;
            player_b->payoff += 3;
        }

        // A va devenir tolérant ou intolérant suivant comment il se compare avec B
        if (player_a->isTolerant != player_b->isTolerant &&
            player_a->payoff < player_b->payoff)
        {
            player_a->isTolerant = player_b->isTolerant;

            tolerant_players += (player_b->isTolerant ? 1 : -1);
        }

        // A va maintenant être jugé par tous les autres joueurs
        for (int other_index=0; other_index<N; ++other_index) {
            Player *other = &players[other_index];

            if (!other->isTolerant) {
                if (player_a->isA != other->isA) {
                    // "Donors of the other class are always judged bad"
                    other->judgesGood[player_a_index] = false;
                } else if (!cooperate && player_a->isA != player_b->isA) {
                    // "Individuals of her own class who do not help individuals of the other class are always judged good"
                    other->judgesGood[player_a_index] = true;
                } else if (cooperate && player_a->isA != player_b->isA) {
                    // "Individuals of her own class who do help individuals of the other class are always judged bad"
                    other->judgesGood[player_a_index] = true;
                } else {
                    // Joueurs de la même classe jugés bons s'ils coopèrent
                    other->judgesGood[player_a_index] = cooperate;
                }
            } else {
                other->judgesGood[player_a_index] = cooperate && other->judgesGood[player_b_index];
            }
        }
    }

    std::cout << "Nombre de joueurs tolérants à la fin du jeu : " << tolerant_players << std::endl;

    // Supprimer les données
    for (int i=0; i<N; ++i) {
        delete[] players[i].judgesGood;
    }

    delete[] players;
}