#include <iostream>
#include <stdlib.h>

static const int N = 1000;
static const int STEPS = 1000000;

struct Players
{
    bool *isA;
    bool *isTolerant;
    int *payoff;
    bool **judgesGood;
};

int main(int argc, char **argv)
{
    (void) argc;
    (void) argv;

    // Créer N joueurs qui jugent tous les autres comme étant bons
    int tolerant_players = 0;
    Players players;

    players.isA = new bool[N];
    players.isTolerant = new bool[N];
    players.payoff = new int[N];
    players.judgesGood = new bool*[N];

    for (int i=0; i<N; ++i) {
        players.isA[i] = rand() & 1;
        players.isTolerant[i] = rand() & 1;
        players.payoff[i] = 0;
        players.judgesGood[i] = new bool[N];

        if (players.isTolerant[i]) {
            tolerant_players += 1;
        }

        // Initialiser les jugements
        for (int j=0; j<N; ++j) {
            players.judgesGood[i][j] = rand() & 3;
        }
    }

    // Simuler le jeu
    for (int t=0; t<STEPS; ++t) {
        // Choisir deux joueurs
        int player_a = rand() % N;
        int player_b = rand() % N;

        // Savoir si le joueur A va coopérer avec le joueur B
        bool cooperate = false;

        if (!players.isTolerant[player_a]) {
            cooperate = (players.isA[player_a] == players.isA[player_b]);
        } else {
            cooperate = (players.judgesGood[player_a][player_b] | players.judgesGood[player_a][player_a]);
        }

        cooperate &= rand() & 7;       // Une chance sur 8 que la coopération échoue

        if (cooperate) {
            players.payoff[player_a] -= 2;
            players.payoff[player_b] += 3;
        }

        // A va devenir tolérant ou intolérant suivant comment il se compare avec B
        if (players.isTolerant[player_a] != players.isTolerant[player_b] &&
            players.payoff[player_a] < players.payoff[player_b])
        {
            players.isTolerant[player_a] = players.isTolerant[player_b];

            tolerant_players += (players.isTolerant[player_b] ? 1 : -1);
        }

        // A va maintenant être jugé par tous les autres joueurs
        for (int other=0; other<N; ++other) {
            if (!players.isTolerant[other]) {
                if (players.isA[player_a] != players.isA[other]) {
                    // "Donors of the other class are always judged bad"
                    players.judgesGood[other][player_a] = false;
                } else if (!cooperate && players.isA[player_a] != players.isA[player_b]) {
                    // "Individuals of her own class who do not help individuals of the other class are always judged good"
                    players.judgesGood[other][player_a] = true;
                } else if (cooperate && players.isA[player_a] != players.isA[player_b]) {
                    // "Individuals of her own class who do help individuals of the other class are always judged bad"
                    players.judgesGood[other][player_a] = true;
                } else {
                    // Joueurs de la même classe jugés bons s'ils coopèrent
                    players.judgesGood[other][player_a] = cooperate;
                }
            } else {
                players.judgesGood[other][player_a] = cooperate && players.judgesGood[other][player_b];
            }
        }
    }

    std::cout << "Nombre de joueurs tolérants à la fin du jeu : " << tolerant_players << std::endl;

    // Supprimer les données
    for (int i=0; i<N; ++i) {
        delete[] players.judgesGood[i];
    }

    delete[] players.isA;
    delete[] players.isTolerant;
    delete[] players.payoff;
    delete[] players.judgesGood;
}